<?php
namespace Zeedhi\DataMining\Server\Controller;

use Zeedhi\DataMining\Common\Util\Environment;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\DataMining\Common\Service\DataCollector;

class Import {

    const PROCESS = 'process';
    const PAGE    = 'page';

    protected $importService;
    protected $environment;

    public function __construct(DataCollector\Import $importService, Environment $environment){
        $this->importService = $importService;
        $this->environment = $environment;
    }

    public function setProcessLength(Request\Row $request, Response $response) {
        try{
            $organizationId = $request->getParameter('organizationId');
        }catch (\Exception $e){
            $organizationId = $request->getParameter('organization');
        }
        $this->environment->setOrganization($organizationId);

        try {
            $row = $request->getRow();
            if (isset($row['length'])) {
                $process = $request->getParameter(self::PROCESS);
                $this->importService->setProcessLength($process, $row['length']);
            } else {
                $response->setCriticalError(new Response\Error("Missing parameters. The process and length are required fields.", 500));
            }
        } catch (\Exception $e) {
            $response->setCriticalError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setProcessLengthNotPage(Request\Row $request, Response $response) {
        try{
            $organizationId = $request->getParameter('organizationId');
        }catch (\Exception $e){
            $organizationId = $request->getParameter('organization');
        }
        $this->environment->setOrganization($organizationId);

        try {
            $row = $request->getRow();
            if (isset($row['length'])) {
                $process = $request->getParameter(self::PROCESS);
                $this->importService->setProcessLengthNotPage($process, $row['length']);
            } else {
                $response->setCriticalError(new Response\Error("Missing parameters. The process and length are required fields.", 500));
            }
        } catch (\Exception $e) {
            $response->setCriticalError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setStartNotPage(Request\Row $request, Response $response) {
        try{
            $organizationId = $request->getParameter('organizationId');
        }catch (\Exception $e){
            $organizationId = $request->getParameter('organization');
        }
        $this->environment->setOrganization($organizationId);

        try {
            $process = $request->getParameter(self::PROCESS);
            $this->importService->setStartNotPage($process);

        } catch (\Exception $e) {
            $response->setCriticalError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setQueryLength(Request\Row $request, Response $response) {
        try{
            $organizationId = $request->getParameter('organizationId');
        }catch (\Exception $e){
            $organizationId = $request->getParameter('organization');
        }
        $this->environment->setOrganization($organizationId);

        try {
            $row = $request->getRow();
            if (isset($row['length'])) {
                $process = $request->getParameter(self::PROCESS);
                $this->importService->setQueryLength($process, $row['length']);
            } else {
                $response->setCriticalError(new Response\Error("Missing parameters. The process and length are required fields.", 500));
            }
        } catch (\Exception $e) {
            $response->setCriticalError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function sendEmailFinalize(Request\Row $request, Response $response) {
        try{
            $organizationId = $request->getParameter('organizationId');
        }catch (\Exception $e){
            $organizationId = $request->getParameter('organization');
        }
        $this->environment->setOrganization($organizationId);

        try {
            $process = $request->getParameter(self::PROCESS);
            $this->importService->sendEmailFinalize($process);
        } catch (\Exception $e) {
            $response->setCriticalError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function addProcessLog(Request\Row $request, Response $response) {
        try{
            $organizationId = $request->getParameter('organizationId');
        }catch (\Exception $e){
            $organizationId = $request->getParameter('organization');
        }

        $this->environment->setOrganization($organizationId);

        try {
            $process = $request->getParameter(self::PROCESS);
            $row = $request->getRow();

            $logMessage = $row['message'];
            $this->importService->addProcessLog($process, 'debug', $logMessage);
        } catch (\Exception $e) {
            $response->setCriticalError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function sync(Request\DataSet $request, Response $response) {
        try{
            $organizationId = $request->getParameter('organizationId');
        }catch (\Exception $e){
            $organizationId = $request->getParameter('organization');
        }
        $this->environment->setOrganization($organizationId);

        try {
            $process = $request->getParameter(self::PROCESS);
            $page    = $request->getParameter(self::PAGE);
            $dataSet = $request->getDataSet();
            $this->importService->sync($organizationId, $process, $page, json_decode(json_encode($dataSet->getRows()), true));
        } catch (\Exception $e) {
            $response->setCriticalError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function syncNotPage(Request\DataSet $request, Response $response) {
        try{
            $organizationId = $request->getParameter('organizationId');
        }catch (\Exception $e){
            $organizationId = $request->getParameter('organization');
        }
        $this->environment->setOrganization($organizationId);

        try {
            $process = $request->getParameter(self::PROCESS);
            $page    = $request->getParameter(self::PAGE);
            $dataSet = $request->getDataSet();
            $this->importService->syncNotPage($organizationId, $process, $page, json_decode(json_encode($dataSet->getRows()), true));
        } catch (\Exception $e) {
            $response->setCriticalError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function notifyError(Request\Row $request, Response $response) {
        try{
            $organizationId = $request->getParameter('organizationId');
        }catch (\Exception $e){
            $organizationId = $request->getParameter('organization');
        }
        $this->environment->setOrganization($organizationId);

        try {
            $process = $request->getParameter(self::PROCESS);

            $row = $request->getRow();
            $message = empty($row['message']) ? NULL : $row['message'];
            $trace = empty($row['trace']) ? NULL : $row['trace'];

            $this->importService->setErrorOnProcess($process, $message, 500, $trace);
        } catch (\Exception $e) {
            $response->setCriticalError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

}
